% Main program for quasi-conformal statistical shape analysis
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi, H. L. Chan, R. Yong, S. Ranjitkar, A. Brook, G. Townsend, K. Chen, and L. M. Lui, 
%     "Tooth morphometry using quasi-conformal theory."
%     Pattern Recognition, 99, 107064, 2020.
%
% Copyright (c) 2019, Hei Long Chan

function [ TPR, TNR, CA, selection ] = QCSSA( parameters, threshold, fold, Energy_cluster1, Energy_cluster2, method, selectionflag, printflag )

if nargin<=5
    method = 'l2';
end
if nargin<=6
    selectionflag = 1;
end
if nargin<=7
    printflag = 1;
end
if fold==1
    fold = 'n';
end

% Compute the energy
E_c1 = QCSSA_RestoreEnergy(parameters,Energy_cluster1);
E_c2 = QCSSA_RestoreEnergy(parameters,Energy_cluster2);

% Extract statistical significant region
if selectionflag
    [selection,~] = QCSSA_Selection( E_c1, E_c2, threshold );
else
    selection = 1:size(E_c1,2);
end
if isempty(selection)
    selection = 1:size(E_c1,2);
end
E_c1 = E_c1(:,selection);
E_c2 = E_c2(:,selection); 

% Cross validation
[ TPR, TNR, CA ] = QCSSA_Classify( E_c1, E_c2, fold, method );

% Print the final result
if printflag
    display(sprintf('*************************************************************************************************'))
    display(sprintf(' '))
    fprintf(' Parameter setting: ')
    for ii = 1:length(parameters)
        fprintf('p%i= %f,    ',ii,parameters(ii));
    end
    fprintf('\n')
    display(sprintf('Significant region: p_cut = %f,   #. of vertices selected = %i',threshold,length(selection)))
    display(sprintf('          Database: #. of cluster 1 = %i,    #. of cluster 2 = %i.',size(E_c1,1),size(E_c2,1)))
    display(sprintf('-------------------------------------------------------------------------------------------------'))
    display(sprintf('            Result: TPR = %f,  TNR = %f,    Accuracy = %f',TPR,TNR,CA))
    display(sprintf(' '))
    display(sprintf('*************************************************************************************************'))
end