function bestchop = separate(A)
% This function finds the best cut for a row vector containing only +1 and
% -1 as its elements, such that the number of +1 on the left of the cut
% plus the number of -1 on the right of the cut is maximized.
%
% Remark: When the position of the best cut is not unique, this function
% reports the left-most possible cut.
%
% Example:
%
% (input)  A = [1,1,1,1,-1,-1,-1,-1]
% (output) bestchop = 4
%
% (input)  A = [1,1,-1,1,-1,-1,-1,-1]
% (output) bestchop = 2

diff = A(1:end-1)-A(2:end);
try
    critical = [0,find(diff~=0),length(A)];
catch
    critical = [0,(find(diff~=0))',length(A)];
end
countlength = critical(2:end)-critical(1:end-1);
if mod(length(countlength),2)==0
    net = [countlength(1),(countlength(3:2:end)-countlength(2:2:end-1)),-countlength(end)];
else
    net = [countlength(1),(countlength(3:2:end)-countlength(2:2:end))];
end
all = tril(ones(length(net)+1))*[0;net'];
pos = find(all==max(all));
countlength = [0,countlength,0];
if length(pos)==1
    bestchop = sum(countlength(1:2*pos-2));
else
    bestchop = zeros(length(pos),1);
    for ii = 1:length(pos)
        bestchop(ii) = sum(countlength(1:2*pos(ii)-2));
    end
end