% Demo for quasi-conformal statistical shape analysis using the proposed method in [1].
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi, H. L. Chan, R. Yong, S. Ranjitkar, A. Brook, G. Townsend, K. Chen, and L. M. Lui, 
%     "Tooth morphometry using quasi-conformal theory."
%     Pattern Recognition, 99, 107064, 2020.
%
% Copyright (c) 2019, Hei Long Chan

addpath('code')

%%
% Load sample data
load('energy.mat')
load('mean_surface.mat');
E1{1}=Energy_H_A;
E1{2}=Energy_K_A;
E1{3}=Energy_d_A;
E2{1}=Energy_H_T;
E2{2}=Energy_K_T;
E2{3}=Energy_d_T;
E3{1}=Energy_H_M;
E3{2}=Energy_K_M;
E3{3}=Energy_d_M;
E4{1}=Energy_H_F;
E4{2}=Energy_K_F;
E4{3}=Energy_d_F;

%% Visualization of energy components
Hs = mean((Energy_H_A+Energy_H_T+Energy_H_M+Energy_H_F)/2)';
Ks = mean((Energy_K_A+Energy_K_T+Energy_K_M+Energy_K_F)/2)';
Ds = mean((Energy_d_A+Energy_d_T+Energy_d_M+Energy_d_F)/2)';
figure
set(gcf, 'Units', 'Normalized', 'OuterPosition', [0, 0.04, 1, 0.96]);
subplot(1,3,1)
patch('Vertices', v', 'Faces', f', 'CData', Hs, 'FaceColor','interp', 'EdgeColor', [0.3,0.3,0.3],'FaceAlpha',1);
view(132,16)
axis equal
axis off
title('Mean curvature difference h_{diff}')
subplot(1,3,2)
patch('Vertices', v', 'Faces', f', 'CData', Ks, 'FaceColor','interp', 'EdgeColor', [0.3,0.3,0.3],'FaceAlpha',1);
view(132,16)
axis equal
axis off
title('Gaussian curvature difference K_{diff}')
subplot(1,3,3)
patch('Vertices', v', 'Faces', f', 'CData', Ds, 'FaceColor','interp', 'EdgeColor', [0.3,0.3,0.3],'FaceAlpha',1);
view(132,16)
axis equal
axis off
title('Teichmuller distance d')
if ~verLessThan('matlab','9.5')
    sgtitle('Visualization of energy components plotted on the mean surface')
end

%% Classification between ancestry
fprintf('\n Classification between ancestry \n')
[~,~,~,S1] = QCSSA([0.191,0.2034,0.9603],0.1,1,E1,E2);
s1 = zeros(length(v),1); s1(S1) = 1;

%% Classification between gender
fprintf('\n\n Classification between gender \n')
[~,~,~,S2] = QCSSA([0.233,0.0147,0.9724],0.001,1,E3,E4);
s2 = zeros(length(v),1); s2(S2) = 1;

%% Visualization of the shape index
e1 = 0.191*mean(E1{1}+E2{1})+0.2034*mean(E1{2}+E2{2})+0.9603*mean(E1{3}+E2{3});
e2 = 0.233*mean(E3{1}+E4{1})+0.0147*mean(E3{2}+E4{2})+0.9724*mean(E3{3}+E4{3});
figure
set(gcf, 'Units', 'Normalized', 'OuterPosition', [0, 0.04, 1, 0.96]);
subplot(1,2,1)
patch('Vertices', v', 'Faces', f', 'CData', e1, 'FaceColor','interp', 'EdgeColor', [0.3,0.3,0.3],'FaceAlpha',1);
view(132,16)
axis equal
axis off
title('Classification between ancestry')
subplot(1,2,2)
patch('Vertices', v', 'Faces', f', 'CData', e2, 'FaceColor','interp', 'EdgeColor', [0.3,0.3,0.3],'FaceAlpha',1);
view(132,16)
axis equal
axis off
title('Classification between gender')
if ~verLessThan('matlab','9.5')
    sgtitle('Visualization of the shape index plotted on the mean surface')
end

%% Visualization of statistical significant region
figure
set(gcf, 'Units', 'Normalized', 'OuterPosition', [0, 0.04, 1, 0.96]);
subplot(1,2,1)
patch('Vertices', v', 'Faces', f', 'CData', s1, 'FaceColor','interp', 'EdgeColor', [0.3,0.3,0.3],'FaceAlpha',1);
view(132,16)
axis equal
axis off
title('Classification between ancestry')
subplot(1,2,2)
patch('Vertices', v', 'Faces', f', 'CData', s2, 'FaceColor','interp', 'EdgeColor', [0.3,0.3,0.3],'FaceAlpha',1);
view(132,16)
axis equal
axis off
title('Classification between gender')
if ~verLessThan('matlab','9.5')
    sgtitle('Highlight of vertices selected under the p-value test')
end