function map = map_reconstruct(v,f,mu,constraint_x,constraint_y)
% Reconstruct the quasi-conformal map with the constraints.
%
% If you use this code in your own work, please cite the following paper:
% [1] G. P. T. Choi, H. L. Chan, R. Yong, S. Ranjitkar, A. Brook, G. Townsend, K. Chen, and L. M. Lui, 
%     "Tooth morphometry using quasi-conformal theory."
%     Pattern Recognition, 99, 107064, 2020.
%
% Copyright (c) 2019, Gary Pui-Tung Choi
% https://scholar.harvard.edu/choi

A = generalized_laplacian(v,f,mu);
B = A;

d1 = -A(:,constraint_x(:,1))*constraint_x(:,2);
d1(constraint_x(:,1)) = constraint_x(:,2);
A(constraint_x(:,1),:) = 0; 
A(:,constraint_x(:,1)) = 0;
A = A + sparse(constraint_x(:,1),constraint_x(:,1),ones(length(constraint_x(:,1)),1), size(A,1), size(A,2));
umap = A\d1;

d2 = -B(:,constraint_y(:,1))*constraint_y(:,2);
d2(constraint_y(:,1)) = constraint_y(:,2);
B(constraint_y(:,1),:) = 0; 
B(:,constraint_y(:,1)) = 0;
B = B + sparse(constraint_y(:,1),constraint_y(:,1),ones(length(constraint_y(:,1)),1), size(B,1), size(B,2));
vmap = B\d2;

map = [umap,vmap];

